@@ -3,10 +3,11 @@ from typing import List, Union
 from airflow import configuration
 from airflow.utils.decorators import apply_defaults
 from airflow.operators import BaseOperator
-from airflow_kubernetes_job_operator.kube_api import KubeObjectState
+from airflow_kubernetes_job_operator.kube_api import KubeResourceState
 from airflow_kubernetes_job_operator.utils import (
     to_kubernetes_valid_name,
 )
+from airflow_kubernetes_job_operator.utils import resolve_relative_path
 from airflow_kubernetes_job_operator.job_runner import JobRunner, JobRunnerDeletePolicy
 from airflow_kubernetes_job_operator.exceptions import KubernetesJobOperatorException
 from airflow_kubernetes_job_operator.config import (
@@ -86,7 +87,9 @@ class KubernetesJobOperator(BaseOperator):
             "body is None, body_filepath is None and an image was not defined. Unknown image to execute."
         )
 
-        body = body or self._read_body(body_filepath or DEFAULT_EXECUTION_OBJECT_PATHS[DEFAULT_EXECTION_OBJECT])
+        body = body or self._read_body(
+            resolve_relative_path(body_filepath or DEFAULT_EXECUTION_OBJECT_PATHS[DEFAULT_EXECTION_OBJECT], 2)
+        )
 
         assert body is not None and (isinstance(body, (dict, str))), ValueError(
             "body must either be a yaml string or a dict"
@@ -125,7 +128,7 @@ class KubernetesJobOperator(BaseOperator):
             body=body,
             logger=self.logger,
             namespace=namespace,
-            auto_load_kube_config=False,
+            auto_load_kube_config=True,
             name_prefix=self._create_job_name(task_id),
             show_pod_logs=get_logs,
             delete_policy=delete_policy,
@@ -229,7 +232,7 @@ class KubernetesJobOperator(BaseOperator):
                 timeout=self._internal_wait_kuberentes_object_timeout,
             )
 
-            if rslt == KubeObjectState.Failed:
+            if rslt == KubeResourceState.Failed:
                 raise KubernetesJobOperatorException(
                     f"Task {self.task_id} failed. See log for kubernetes execution details."
                 )
